### A Pluto.jl notebook ###
# v0.20.4

using Markdown
using InteractiveUtils

# ╔═╡ a145e2fe-a903-11eb-160f-df7ea83fa3e6
begin
	using HypertextLiteral, PlutoUI, PlutoTeachingTools, PlutoPlotly
	using CSV, DataFrames, Dates, PeriodicalDates
end

# ╔═╡ 10d33d47-0727-4c01-acdb-71949077bd2e
html"""
<picture>
  <source srcset="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/negative/rgb_ibs_en_horizontal_negative.png" media="(prefers-color-scheme: dark)" width="240px!important" style="float:right;">
  <img src="https://gci.iscte-iul.pt/2022/identidade/files/brand_v2/escolas/rgb/ibs/en/horizontal/main/rgb_ibs_en_horizontal_main.png" width="240px" style="float:right;">
</picture>
"""

# ╔═╡ 943edb36-acc8-4b2a-94ab-c544f9eb279b
md"""

# Week 6 - The Central Bank & Monetary Policy Tools

## Exercises

**Macroeconomics, ISCTE-IUL**

"""

# ╔═╡ 1245aef9-18ce-42f5-af82-c2e6cce70798
md"""
**Vivaldo Mendes, Ricardo Gouveia-Mendes, Luís Clemente-Casinhas**

**March 2025**
"""

# ╔═╡ 55188a89-8172-4bb9-9569-765a9f277d0f
md"""
---
"""

# ╔═╡ 4618a663-f77e-4ccc-9b00-e84dbd5beb34
md"""

### Packages used in this notebook

"""

# ╔═╡ 72d45107-22d6-4663-a9b8-c540a4c7411b
md"""
---
"""

# ╔═╡ 116306a9-5f2d-4b96-9124-584dc9835a8a
md"""
## Exercise 1. The central bank´s balance sheet
"""

# ╔═╡ 19741208-c2bd-455a-a19f-74521a8c56bf
md"""
_From the textbook_
"""

# ╔═╡ 9877083b-a80c-4069-89e4-84638fe6ecbb
md"""
Tables 1.1 and 1.2 cover the balance sheet of the Fed (Federal Reserve Board) for 2007 and 2009.

"""

# ╔═╡ 0be26b61-30ce-4128-b2b1-b4eb955244d1
aside(hint(md"```math
\begin{aligned}
&\text { Assets - Central Bank }\\
&\begin{array}{lr}
\hline \text { Items } & \$ \text { billions } \\
\hline \text { Foreign Reserves } & 0 \\
\text { Securities } & 0 \\
\text { Loans } & 0 \\
\text { Other assets } & 0 \\
\text { Total assets } & 0
\end{array}
\end{aligned}
```") ,v_offset=60)

# ╔═╡ cfc72d68-3d45-4673-81a2-7032896fbe89
md"""
$$\begin{aligned}
		&\text { Table 1.1$-$Federal Reserve Board: Assets (\$ billions) }\\
		&\begin{array}{llrr}
			\hline \hline \text {Code} & \text {Items} & \text {2009} & \text {2007} \\
			\hline 
			{tre} & \text {Treasuries*}                 & \qquad \quad  776 & \qquad \quad 797 \\
			{mbs} & \text {Mortage-backed securities*}  & \qquad \quad  801 & \qquad \quad   0 \\
			{fad} & \text {Federal Agency debt*}        & \qquad \quad  258 & \qquad \quad   0 \\
			{oa} & \text {Other Assets}                & \qquad \quad   91 & \qquad \quad  14 \\
			{gol} & \text {Gold}                        & \qquad \quad   12 & \qquad \quad  11 \\
			{fa} & \text {Foreign Assets}            & \qquad \quad   25 & \qquad \quad  47 \\
			{loa} & \text {Loans}                       & \qquad \quad  276 & \qquad \quad  25 \\
			ass & \text {Total Assets}                & \qquad \quad 2239 & \qquad \quad 894 \\
			\hline
\end{array}
\end{aligned}$$
$^* \text{All these items are Securities}$
"""

# ╔═╡ be5df6ed-9417-4ed0-862c-8f337b80084e
aside(hint(md"```math
\begin{aligned}
&\text { Liabilities - Central Bank }\\
&\begin{array}{lr}
\hline \text { Items } & \$ \text { billions } \\
\hline \text { Currency } & 0 \\
\text { Reserves } & 0 \\
\text { Government account } & 0 \\
\text { Other liabilities } & 0 \\
\text { Equity } & 0 \\
\text { Liability + Equity } & 0
\end{array}
\end{aligned}
```") ,v_offset=0)

# ╔═╡ f0a318ef-47b0-4fa3-aeb1-e7d1548fd214
md"""
$$\begin{aligned}
		&\text { Table 1.2$-$Federal Reserve Board: Liabilities (\$ billions)}\\
		&\begin{array}{llrr}
			\hline \hline \text {Code} & \text {Items} & \text {2009} & \text {2007} \\
			\hline 
			cur & \text {Currency}                     & \qquad \qquad  883 & \qquad \quad 792 \\
			gov & \text {Government Account}           & \qquad \qquad  149 & \qquad \quad   5 \\
			res & \text {Reserves of commercial banks} & \qquad \qquad 1073 & \qquad \quad  11 \\
			ol & \text {Other liabilities}            & \qquad \qquad   81 & \qquad \quad  49 \\
			lia &\text {Total liabilities}            & \qquad \qquad 2186 & \qquad \quad 857 \\
			\hline
		\end{array}
	\end{aligned}$$
"""

# ╔═╡ b189ebff-1493-4d95-98d2-99d6f55884fe
md"""

The statistical information above is passed to the notebook, but only that related to 2009 (the index `_9` is used to refer to 2009 data). It would help if you tried to do the same thing for 2007.

"""

# ╔═╡ c99c90e1-e91f-4682-a190-495d2cbd0b06
tre_9 = 776 ; mbs_9 = 801 ; fad_9 = 258 ; oa_9 = 91 ; gol_9 = 12 ; fa_9 = 25 ; loa_9 = 276 ; ass_9 = 2239 ;

# ╔═╡ 98e7e1ad-2964-41e5-9162-cd077978ee15
cur_9 = 883 ; gov_9 = 149 ; res_9 = 1073 ; ol_9 = 81 ; lia_9 = 2187 ;

# ╔═╡ 39480dd4-4e0e-4e5f-9a5c-74f426b9193a
md"""
👉 a) What main monetary aggregate is missing from the data?
"""

# ╔═╡ f83dd4f0-0312-4bb0-98b0-67eeab7177e4
md"""
!!! hint

	To complete the balance sheet, we must calculate the total amounts of `Foreign Reserves` and `Securities`. 

	We will do it only for the year 2009. Students are advised to do the same for 2007.

"""

# ╔═╡ 4bf99794-a645-40c7-b51b-128fce7814df


# ╔═╡ baab71f8-e0be-43b5-83f4-708e4e575ebc


# ╔═╡ 719dbca7-f3f1-4827-bfba-d9813979c42a
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ cdeb5418-39ca-402c-81fd-1a4e42c2e105
md"""
👉 b) Calculate the value of `Foreign reserves` and `Securities`.

"""

# ╔═╡ 02126494-1ba6-47f8-8c10-9c4d34e20d57
md"""
!!! answer "Answer (b)"

	Foreign reserves = ???  ; Securities = ???

"""

# ╔═╡ b7749c4f-ade7-46ec-a95f-ea1349b86956
md"""
👉 c) Complete the balance sheets according to the structure provided in the slides.
"""

# ╔═╡ 84cee9af-0999-4659-a537-85c89f9182cc
md"""
!!! answer "Answer (c)"

	The Fed's balance sheet for the year 2009 is as follows:

	Here        

	Do the same for the year 2007.

"""

# ╔═╡ acce2747-50a8-4531-a4e1-d7bd971f2210
md"""
👉 d) The data from 2007 reflects the Fed's balance sheet before the financial crisis blew up in 2008. At the end of 2008, the Fed started a type of policy known as "Quantitative Easing", and this new policy is already visible in the balance sheet of 2009. What are the main differences between the two years? 
"""

# ╔═╡ c144c3fe-d49f-4243-bdc2-34ba8af88673
md"""
!!! answer "Answer (d)"

	Here

"""

# ╔═╡ 1a073295-059c-4815-b3bd-450f5f88c8d1
md"""
---
"""

# ╔═╡ 99fa722b-bef4-48d7-bd4a-f0459f15eed3
md"""
## Exercise 2. Fed operations and the Monetary Base
"""

# ╔═╡ f4706381-35df-44dd-aea2-83220a94c9d4
md"""

Write down in the balance sheet of the central bank the following operations:

👉 (1) The Fed sells 200 million dollars of bonds denominated in Renminbi.

👉 (2) The Fed buys 600 million of Mortgage-Backed Securities.

👉 (3) The Fed buys 900 million of 10-Year Treasury bonds.

👉 (4) The Fed lends 500 million to commercial banks.

👉 (5) Due to an unexpected event, the private sector withdraws 200 million from bank accounts.

👉 (6) The Fed decided to raise the IOER by 0.25 percentage points.

👉 (7) The Fed buys 400 million dollars of bonds denominated in Euros.

"""

# ╔═╡ 8eeea220-e319-45da-a774-7cf1475f2424
aside(danger(md"The Fed sells 200 million dollars of bonds denominated in Renminbi."),v_offset=100)

# ╔═╡ 4f82c4f5-dab7-4ef4-a1c6-bef80ac2fe64
md"""
!!! answer "Answer (1)"

	The Fed's balance sheet will change as follows:

	Here
	

	The commercial banks' consolidated balance sheet will change as follows:

	Here

	This operation will reduce the Monetary Base by 200 million dollars.

"""

# ╔═╡ 71bbd822-914c-4ea6-93c5-0b3ad86000ca
aside(danger(md"The Fed buys 600 million of Mortgage-Backed Securities."),v_offset=100)

# ╔═╡ 10234ea1-6528-4fd2-bdae-3ca5a36e405b
md"""
!!! answer "Answer (2)"

	The Fed's balance sheet will change as follows:

	Here
	

	The commercial banks' consolidated balance sheet will change as follows:

	Here

	This operation will increase the Monetary Base by 600 million dollars.

"""

# ╔═╡ 76d208b1-80b7-4a8c-96c6-2de15b5e89e8
aside(danger(md"The Fed buys 900 million of 10-Year Treasury bonds."),v_offset=45)

# ╔═╡ 1254cc13-8ac0-468c-bbc5-e838018fa1de
md"""
!!! answer "Answer (3)"

	Here

	Here

"""

# ╔═╡ e7f794fa-ede6-4391-b703-5488fd739f29
aside(danger(md"The Fed lends 500 million to commercial banks."),v_offset=100)

# ╔═╡ 15b3a15f-a0a0-4623-b6ec-43dc33bbe1e3
md"""
!!! answer "Answer (4)"

	The Fed's balance sheet will change as follows:

	Here
	

	The commercial banks' consolidated balance sheet will change as follows:

	Here

	This operation will increase the Monetary Base by 500 million dollars.

"""

# ╔═╡ d2a12416-9d54-479c-956d-2d8869024727
aside(danger(md"The private sector withdraws 200 million from bank accounts."),v_offset=100)

# ╔═╡ 3323975f-e90c-4a39-87d4-2ae9c676e24a
md"""
!!! answer "Answer (5)"

	The Fed's balance sheet will change as follows:

	Here
	

	The commercial banks' consolidated balance sheet will change as follows:

	Here

	This event will not affect the Monetary Base.

"""

# ╔═╡ a4aecd27-1376-4f74-98b6-10502c93e1d0
aside(danger(md"The Fed decided to raise the IOER by 0.25 percentage points."),v_offset=50)

# ╔═╡ d865e28a-5db4-4b39-a7e6-353d380b9e1a
md"""
!!! answer "Answer (6)"

	Here

"""

# ╔═╡ 32c928dd-ca87-4db5-aa28-222e43a7a3cb
aside(danger(md"The Fed buys 400 million dollars of bonds denominated in Euros."),v_offset=50)

# ╔═╡ 5d7ad15e-3057-43c8-913c-67f6a71338b7
md"""
!!! answer "Answer (7)"

	Here

"""

# ╔═╡ f54d0589-0567-4b75-91f2-d9bf223c5fb6
md"""
---
"""

# ╔═╡ 2bd8b31a-8fa2-4404-816e-158df0afcc7d
md"""
## Exercise 3. The Money Multiplier
"""

# ╔═╡ 30f40a81-32cb-4d2b-b3c5-b0b155648283
md"""
The following figure presents the evolution of the USA economy's Monetary Base (MB) and Money Supply (M2) series between 1959 and 2024.

👉 a) What is the MB? And what is the M2?
"""

# ╔═╡ 2c3766cf-1349-4e99-a3fd-62ad6e5fe18a
md"""
!!! answer "Answer (a)"

	MB is ...

	M2 is ...

"""

# ╔═╡ d6a5e159-8b97-4660-8e00-cd251e5d0216
md"""
👉 b) What is the money multiplier?
"""

# ╔═╡ c5363210-26f9-40b1-9a1a-9ac80ffac890
md"""
!!! answer "Answer (b)"

	The money multiplier is ...

"""

# ╔═╡ 3193d821-a4e1-404f-a57e-8c471714bb9d
md"""
👉 c) Looking at the following plot below, what can we conclude about the relationship between M2 and the MB over the period under consideration?
"""

# ╔═╡ 624e0704-a138-4ddd-8086-078d98a84a41
md"""
!!! answer "Answer (c)"

	Here

"""

# ╔═╡ e7fd66aa-cb38-4b7f-92af-aa4fd15549b2
md"""

👉 d) What is the main implication for managing monetary policy that arises from large instability in the money multiplier?


"""

# ╔═╡ fe0e5144-02b9-4df3-84ad-3298a8703baa
md"""
!!! answer "Answer (d)"

	Here

"""

# ╔═╡ 6e20a099-8d46-4a35-83ce-e62c977a6b3d
md"""
---
"""

# ╔═╡ 65da15c3-dcad-4e38-b0aa-bf9dd456285d
md"""
## Exercise 4. Successful Fed
"""

# ╔═╡ 74529060-483d-4afe-8d58-155c5f0b4b1e
md"""

Using the figure below, explain why the Fed so successfully guarantees that short-term market interest rates stay in the range defined by this central bank.

"""

# ╔═╡ ced280ff-d8c1-414e-8855-387727d5dfe2
Resource("https://vivaldomendes.org/images/depot/Tools.png",:width=>600)

# ╔═╡ 85cb81f0-8e2f-4142-9fdc-b8839d2a6cea
md"""
!!! note

	Starting July 29, 2021, the Fed replaced the interest rate on excess reserves (IOER) with the interest rate on reserve balances (IORB). Despite the change in the name, the concept is basically the same. That is why we keep the acronym used by the textbook.
"""

# ╔═╡ 5e6f355c-cb29-416f-8799-0234fbc87f1f
md"""
!!! answer

	Here

"""

# ╔═╡ 5bb6a994-ac2d-44cd-89c9-c3c3fbdca100
md"""
---
"""

# ╔═╡ 14396951-fcfa-41eb-822a-f673cd830565
md"""
## Exercise 5. The Target Rate and the Fed Funds Rate
"""

# ╔═╡ 6f384fbf-b581-4eb8-9ce4-026714d99537
md"""
The following figure plots the evolution of the Fed Target Rate Range (upper limit and lower limit) and the Fed Funds Rate (FFR).

"""

# ╔═╡ 10417add-96dd-427b-825a-dc558f710807
md"""

👉 **a)**  From those two rates above (Target Rate and the Fed Funds Rate), the Fed can set which of them by law?

"""

# ╔═╡ 3e9327e9-b113-4468-b620-4ae8552c214d
md"""
!!! answer "Answer (a)"

	Here
"""

# ╔═╡ 8007d3dd-d6e1-405c-9018-56cd30004af7
md"""
👉 **b)** What is the Fed Funds Rate? Does the Fed directly set it?

"""

# ╔═╡ b7cb4d68-1e36-4d1f-acaa-0855d2127ef8
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ 7641617b-48f0-4ec2-8504-3d655d0134ad
md"""
👉 **c)** The figure above covers the period from 2009 onwards. This period has been tremendously unstable (economically, financially, and politically). Do you find in that figure favorable evidence that the Fed has a remarkable capacity to influence short-term interest rates? Explain.

"""

# ╔═╡ b0b75354-1c37-4f06-afe3-5b1d420ae245
md"""
!!! answer "Answer (c)"

	Here

"""

# ╔═╡ 164e38a5-1368-4456-a909-dbe8677ebcb8
md"""
👉 **d)** Using the zoom functionalities of our plots, zoom in on the period from the start of 2023. Can you tell me what the last day was when there was a change in the target rate by the Fed?

"""

# ╔═╡ 4550ba42-1a49-4fa9-932b-e4d34210233f
md"""
!!! answer "Answer (d)"

	Here

"""

# ╔═╡ 88bab2ef-32ba-49b0-a5e0-36dad20e39e2
md"""
👉 **e)** Usually, the Fed makes three types of rate decisions: (i) keep rates unchanged, (ii) change them in a very aggressive manner, (iii) change them gradually (softly). Can you provide evidence about periods when the Fed did so? 
"""

# ╔═╡ 869004a1-039e-41e4-a61f-453c25406ddc
md"""
!!! answer "Answer (e)"

	Here

"""

# ╔═╡ f4fe404c-ca57-4682-8a51-3c08abedfb36
md"""
---
"""

# ╔═╡ 60e62656-e863-4595-9d94-4e7d0cf6d935
md"""
## Exercise 6. The Fed's dilemma
"""

# ╔═╡ 88675aac-0d69-4028-aee1-c0e0c22ab5e2
md"""

The US has experienced the highest inflation rates for the last forty years since early 2021. The following figure presents the evolution of the two main measures of inflation (Headline inflation and Core inflation) and the Fed Funds Rate since 2018. "Core" inflation (which excludes energy and food prices from Headline inflation) is one of the preferred Fed measures of inflation.

>_"We have got to get inflation behind us. I wish there were a painless way to do that. There isn't."_ Jerome Powel, Chair of the Fed, 21 September 2022 

What does Jerome Powel mean by saying, "_I wish there were a painless way to do that_"?
"""

# ╔═╡ 2283145f-b909-4b23-8292-4bb660f303d1
md"""
!!! hint

	**_Headline inflation_** is the usual term for inflation. It includes all goods and services used to measure it. 

	**_Core inflation_** excludes food and energy goods from its measure because the prices of these two goods are very volatile in the short term.

	Both measures showed signs of increasing steadily until the Fed started raising interest rates aggressively. 
"""

# ╔═╡ 08bf1683-0c0d-4c52-a446-6f8be4135aed
md"""
!!! answer

	Here

"""

# ╔═╡ d08dc62f-8951-4e36-8818-c325316ece23
md"""
---
"""

# ╔═╡ bf337699-7647-4538-85e1-ed2fd9d05269
md"""
## Exercise 7. A 2022 1M-dollar question: how far would the Fed go?
"""

# ╔═╡ 5b1919fb-f5f5-4682-a1e0-1c767825c902
md"""

In January 2021, Headline inflation was 1.4%; it jumped to 8.5% in May 2022 and showed no signal of changing course. Everybody was under the impression that inflation was getting out of control, and the New York Times, 21 September 2022, made the following point: 

>_"The Fed raised borrowing costs by three-quarters of a percentage point, their third-straight jumbo increase, taking their official interest rate to a range of 3 to 3.25 percent. But they also penciled in additional increases for the rest of this year [2022] and next [2023], projecting that rates would reach 4.4 percent by the end of 2022 and climb to 4.6 percent by the end of 2023."_ 

"""

# ╔═╡ baf57f6e-c2db-49f5-b65e-11656176372c
md"""
👉 a)  How far would we expect the Fed to go, given the pace of inflation then and the evidence from previous surges in inflation (see the following figure) since 2000? **This was a one-million-dollar question in the summer of 2022!**
"""

# ╔═╡ 1d323377-a98c-4a93-977f-7796acede851
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ ba7a9892-dc53-4c9a-867f-17d0812ec97a
md"""
👉 b) In September 2022, the Fed projected that the FFR would reach 4.4 percent by the end of 2022 and climb to 4.6 percent by 2023. Did the Fed's forecast perform well in both years? (check the values in January 2023 and in January 2024) 
"""

# ╔═╡ c3ab833b-0d82-40b3-9553-d90c32c369d8
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ 88a9d07a-d222-46b3-bcae-d19e4ac78017
md"""
👉 c) Suppose you were in December 2023. Do you want to have a go and provide a forecast for the next year [2024]? Should the Fed raise rates to 6% or 7% during 2024? should the Fed reduce rates?
"""

# ╔═╡ c93efdd7-1ae9-4989-b5cb-cfdb4347086d
md"""
!!! answer "Answer (c)"

	Here

"""

# ╔═╡ 8f643bb9-8826-421f-8590-b440c65101eb
md"""
---
"""

# ╔═╡ dca53198-44ed-402e-8a68-401a6006d1f9
md"""
## Exercise 8. Oil prices & Fed mistakes
"""

# ╔═╡ 2b7f9399-1606-4374-abfb-abf9e7ab2e90
md"""

David Kelly, JPMorgan Funds chief global strategist, in 10 October 2022 argued that:

>_"In the long history of Federal Reserve mistakes, one general error stands out. They tend to wait too long and then do too much, and, in so doing, actually accentuate rather than tame the business cycle. They appear to be well on their way to repeating this error today."_ 

👉 **a)**  Is monetary policy a science?

"""

# ╔═╡ 0223f27f-532b-4b75-af02-817fe5f44ed6
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ 268c2d4f-0211-4ff5-a054-5dbe4c4b3078
md"""
👉 **b)** In the two figures below, we plot the annual growth rates of crude oil prices (Oil) and the rate of Headline inflation (CPI) since 1998. The first plot is a time series of the two growth rates, while the second is a cross-plot. Do you see any link between the growth rate of oil prices and the inflation rate for the US economy for that period?
"""

# ╔═╡ b04f7af0-7e7c-45a5-ad82-78e866348567
md"""
!!! answer "Suggested answer (b)"

	Here

"""

# ╔═╡ d710baca-f026-4e3d-926c-145594eedc81
md"""
👉 **c)** What is the implication of oil price increases for monetary policy management?
"""

# ╔═╡ 1a8a1ae4-6ecd-4a91-a482-3024ca11fb98
md"""
!!! answer "Suggested answer (c)"

	Here

"""

# ╔═╡ 69621e4b-5163-40a4-a70c-429d8afe6ff5
md"""
---
"""

# ╔═╡ c842b2da-1bf5-43db-9bc0-a8af8be64b9c
md"""
## Exercise 9. Monetary Base vs Total Assets (optional)
"""

# ╔═╡ 7e7e4292-dfc8-41b4-a5a0-d7646f8fb495
md"""

The following figure plots the evolution of two important monetary aggregates for the US economy—`Total Assets` and the `Monetary Base` (MB)—between 2003 and August 2022. By inspecting the plot (use the zoom facility), analyze the behavior of these two crucial macroeconomic aggregates in three different periods (before 2008, between 2008 and 2019, and after 2019), considering the gap between these two aggregates.

"""

# ╔═╡ e547d364-216c-4165-982f-1e04c9a0945c
md"""
!!! answer "Suggested answer"

	Here	

"""

# ╔═╡ 01875faf-3231-4bb8-899d-0dd4e569ddc9
md"""
## Exercise 10. COVID19 and monetary policy (optional)
"""

# ╔═╡ 63efdb6e-dff1-420f-a99d-90cece86bac2
md"""
In Table 9.1, we present the data on August 11, 2021, of the balance sheet of the Federal Reserve Board. This data was compiled from [FRB](https://www.federalreserve.gov/releases/h41/current/h41.htm) on August 13. This table reflects the drastic intervention of the Fed in the financial markets to fight the negative impacts of the COVID19 pandemic. What are the primary aggregates that are, in essence, new in this balance regarding the standard balance sheet of the Fed?

"""

# ╔═╡ 1a1d2e86-9436-4fe3-8dbd-74509f439a98
md"""
$$\begin{aligned}
& \text {Table 9.1 $-$ Balance Sheet of the Federal Reserve Board: August 11, 2021}\\
&\begin{array}{lr|lr}
\hline \hline \text {\textbf{Assets}} & \text{\$ millions} & \text{\textbf{Liabilities}} & \text{ \$ millions} \\
\hline 
\text{Foreign reserves} & \qquad \quad  38611   & \text{Currency}                & \qquad \quad  2139815\\
\text{Securities}       & \qquad \quad  8028859 & \text{Reserves}                & \qquad \quad  4369073 \\
\text{Loans}            & \qquad \quad  82500   & \text{Reverse Repo Agreements} & \qquad \quad  1267998 \\
\text{Portfolio}        & \qquad \quad  63707   & \text{Government account}      & \qquad \quad  389747 \\
\text{Other assets}     & \qquad \quad  43480   & \text{Other liabilities}       & \qquad \quad  50790 \\
		                &           			& \text{Equity}                  & \qquad \quad  39734 \\
\text{Total assets}     & \qquad \quad 8257157  & \text{Liabilities+Equity}    & \qquad \quad  8257157 \\
\hline
\end{array}
\end{aligned}$$
"""

# ╔═╡ 9c396c2d-9512-4fab-a49f-7883ddb1d98c
md"""
!!! answer 

	Here

"""

# ╔═╡ a5654312-d9e7-4d86-8663-6e4b4ea4d9b7
md"""
---
"""

# ╔═╡ dde1d26c-da2e-46ad-85cb-fcb03dc4b126
md"""
## Exercise 11. QE and financial markets (not covered)
"""

# ╔═╡ 5ba94526-2197-42ff-a569-038a07e83840
md"""

In 2008, the US economy was struck by the dippest economic and financial crisis since the 1930s, known as the "Great Recession". As a response, the Fed dropped nominal interest rates to zero and implemented an **_unconventional_** monetary policy known as Quantitative Easing (QE). One of the most dramatic impacts of a big financial crisis is the collapse of financial indicators. The figure below plots a well-known financial indicator, the "_Shiller PE Ratio_", for the US economy and when the Fed implemented QE. What could one conclude about the efficacy of this type of unconventional monetary policy tool in avoiding the collapse of the financial market due to the negative impact of the Great Recession?

"""

# ╔═╡ 660066db-af49-4473-aeff-161aedee0aba
md"""
!!! note

	The Shiller PE Ratio (also known as CAPE) is one of the most used instruments to assess the expected future returns from equities over timescales of 10 to 20 years. It is calculated using the US S&P 500 equity market index divided by the average of ten years of earnings (moving average), adjusted for inflation. This ratio should remain relatively stable over time with stable productivity and inflation. Robert Shiller was awarded the Nobel Prize in Economics in 2013 for his work on empirical analysis of asset prices.
"""

# ╔═╡ 06beba5f-e3af-4649-8d19-48a7036ebe01
md"""
!!! answer "Suggested answer"

	Here

"""

# ╔═╡ 98338f07-e909-4c20-b2db-bf561db17b81
md"""
---
"""

# ╔═╡ ed79aa0d-e286-40dd-aa8f-19804ca94b55
md"""
## Exercise 12. QE and inflation (not covered)
"""

# ╔═╡ 94c6354c-4f89-4982-8d18-677250f6380d
md"""
👉 **a)** Taking the tools of unconventional monetary policy in mind, why do you think the Fed's former Chair Ben Bernanke, who led the Fed through the storm of the Great Recession, remarked in his most recent book "The Courage to Act: A Memoir of a Crisis and Its Aftermath", 2017, Norton, N.Y.

>*"As I had often remarked, monetary policy is 98 percent talk and 2 percent action. That’s especially true when short-term interest rates hover close to zero and influencing expectations about future interest rates becomes critically important."*
"""

# ╔═╡ def794bb-0c9e-4132-9300-d597fb29fd64
md"""
!!! answer "Answer (a)"

	Here

"""

# ╔═╡ 10dad751-127a-44b3-a3cb-a22644049964
md"""

👉 **b)** In [Ben Bernanke (2020),](https://www.brookings.edu/blog/ben-bernanke/2020/01/04/the-new-tools-of-monetary-policy/) "The new tools of monetary policy", American Economic Association Presidential Address, January 4, 2020, Bernanke states that:

>*"Consistent with their declared “symmetric” inflation targets, the Federal Reserve and other central banks should defend against inflation that is too low as least as vigorously as they resist inflation that is modestly too high.  Although the new monetary tools have proved their worth and can be made more effective in the future, keeping inflation and inflation expectations close to target is critically important for preserving or increasing available policy space."*



Using the following figure, what can one conclude about the efficacy of Quantitative Easing (QE) tools, regarding "keeping inflation and inflation expectations" close to target?
"""

# ╔═╡ 449374a2-0df7-4b8c-9286-79764bc5529f
md"""
!!! answer "Answer (b)"

	Here

"""

# ╔═╡ 45b82b1a-262c-4668-879a-998acd7e160a
md"""
---
"""

# ╔═╡ 09b4be5f-cb7e-4ffa-bbe2-a4e5b2d4fe66
md"""
## Auxiliary cells (please do not delete)
"""

# ╔═╡ 9fa87191-7c90-4738-a45a-acd929c8bd1b
  TableOfContents()

# ╔═╡ 7c178839-3e3f-4013-8351-86efde9d1b5b
md"""
##### Data files used in this notebook

- Assets.csv
- Multiplier.csv
- EFFR.csv
- How_Far.csv
- Oil_CPI.csv
- QE.csv
- Shiller_PE_Ratio.csv
- FedRateRange_MonthlyData.csv

These CSV files must be kept in the same folder as this notebook.
"""

# ╔═╡ 953fc3de-48af-42f6-946b-2225618c093b
md"### CSS code"

# ╔═╡ ccb36e23-9fa5-43c4-a451-275609b4892b
html"""
<style>
	@media screen {
		main {
			margin: 0 auto;
			max-width: 1400px;
    		padding-left: max(100px, 5%);
    		padding-right: max(383px, 10%); 
            # 383px to accomodate TableOfContents(aside=true)
		}
	}
</style>
"""

# ╔═╡ 4c805ee4-e91e-4d88-bffc-e19e42868ef5
begin
      struct TwoColumns{L, R}
    left::L
    right::R
      end
      
      function Base.show(io, mime::MIME"text/html", tc::TwoColumns)
          write(io, """<div style="display: flex;"><div style="flex: 48%;margin-right:2%;">""")
          show(io, mime, tc.left)
          write(io, """</div><div style="flex: 48%;">""")
          show(io, mime, tc.right)
          write(io, """</div></div>""")
      end
end

# ╔═╡ cb9d826b-b609-49d5-ad60-a15efc8cce1d
md"""
### Exercises
"""

# ╔═╡ e7a2be16-2b78-4527-b799-6c208854ff5a
md"""
#### Supporting cells: exercise 3 (Money multiplier)
"""

# ╔═╡ cd8e9140-5696-4019-b208-d6430e68f978
MM = CSV.read("Multiplier.csv", DataFrame);


# ╔═╡ a9095027-f033-4253-9269-105e51887424
begin
	
fig4_1 = plot(MM.Months, [MM.M2 MM.MB])

relayout!(fig4_1, height = 450,  Layout(hovermode="x", title_text="MB vs M2 in the USA (1959--2024): Billions of Dollars", title_x = 0.5, titlefont_size="17"))

restyle!(fig4_1, 1:2, name = ["M2", "MB"], line_color = ["Blue" , "red"], line_width="2.5")
	
fig4_1

end

# ╔═╡ 1471566f-469f-421b-8671-abc49e040bc1
begin
	trace6_7 = scatter(;x = MM.Months, y = MM.M2./MM.MB, 
				name = "M2", mode="lines",
				marker_symbol="circle", marker_size="4",line_width= 2.2, 
            	marker_color = "Blue")
	
	
	layout6_7 = Layout(;
			height = 450,
			title_text = "The Money Multiplier for the USA (1959--2024)", title_x=0.5,
		
			hovermode="x",		
		
            xaxis = attr(
               title = "Monthly obervations"
               #tickformat = "%Y",
               #hoverformat = "%Y-M%m",
               #tick0 = "1989/09/01",
               #dtick = "M60" 
			   ),
		
        	#xaxis_range = [2000, 2020],
        	yaxis_title = "Percentage points",
        	#yaxis_range=[-2, 2], 
        	titlefont_size = 16)

	p6_7 = plot([trace6_7], layout6_7)
end

# ╔═╡ f05055c8-cb5f-4d28-91c0-d84d103efe43
md"""
#### Supporting cells: exercise 5 (The Target rate and the FFR)
"""

# ╔═╡ 5d20e28c-8365-470b-9c41-0471e2c65ad8
EFFR = CSV.read("EFFR.csv", DataFrame);

# ╔═╡ 74d6dd0f-8cf0-45dc-8f5a-250560e8a349
let

    # Define the traces for the upper limit, lower limit, and the effective rate
    trace_upper = scatter(; x=EFFR.Weeks, y=EFFR.UpperLimit, mode="lines", 
                          line_color="blue", name="Upper Limit: Target Rate Range")

    trace_lower = scatter(; x=EFFR.Weeks, y=EFFR.LowerLimit, mode="lines", line_width = "0", 
                          line_color="maroon", name="", 
                          fill="tonexty",
						  fillcolor="rgba(131, 90, 241, 0.8)"
						  #fillcolor="rgba(173, 216, 230, 0.99)"
	) # Light blue fill

    trace_effr = scatter(; x=EFFR.Weeks, y=EFFR.EFFR, mode="lines", 
                         line_color="orange", name="Federal Funds Rate")

	trace_lower2 = scatter(; x=EFFR.Weeks, y=EFFR.LowerLimit.-0, mode="lines", 
                          line_color="black", name="Lower Limit: Target Rate Range")

    # Combine all traces into the figure
    fig05 = plot([trace_upper, trace_lower, trace_lower2, trace_effr])

    # Adjust the layout
    relayout!(fig05,
        height=500,
        title_text="The Fed Target Rate Range and the Fed Funds Rate",
        title_x=0.5,
        hovermode="x",
        legend=attr(x=0.05, y=0.95),
        yaxis_title="Percentage points",
        xaxis=attr(
            title="Weekly observations"
            # Additional axis settings can be uncommented and customized as needed
        )
    )

    # Display the figure
    fig05
end

# ╔═╡ a5c866f0-3546-4934-ae84-9de9bc8e7f64
md"""
#### Supporting cells: exercise 6 (Fed's dilemma)
"""

# ╔═╡ b2d64bfc-2fbd-4677-8a17-dad2cc94b8e9
How_Far= CSV.read("How_Far.csv", DataFrame);

# ╔═╡ 642da752-5c76-482e-97e5-a9964e894868
begin
	fig6_1 = plot(How_Far.Months, [How_Far[:,2] How_Far[:,3] How_Far[:,4] ], line_width = 2, Layout(hovermode ="x", height = 450, xaxis_range=["2018-01-01", "2023-09-01"],
	title_text = "Inflation and the Fed Funds Rate (FFR) for the USA (2018-2022)", title_x=0.5,))
	restyle!(fig6_1, 1:3, name=["Head" "Core" "FFR"], line_color = ["blue" , "red" , "gray"])
	fig6_1
end


# ╔═╡ a8dbf8b7-1f6a-48c1-b8e8-c73062f72277
begin
	fig5_5 = plot(bar(; x= How_Far.Months,  y =  How_Far.FFR - How_Far.Core) , Layout(height = 450, hovermode ="x", 	title_text = "The Fed Funds Rate minus Core Inflation (USA 2000-2023)", title_x=0.5, xaxis_range =["2000-01-01", "2023-11-01"]))
end


# ╔═╡ 202b8d36-8f6f-40ad-a173-e9213996ea5e
md"""
#### Supporting cells: exercise 7 (A 2022 1_Million...)
"""

# ╔═╡ fd598141-c9e6-402d-97f9-bab7e9f8b4dc
EFFR_Montly=CSV.read("FedRateRange_MonthlyData.csv", DataFrame);

# ╔═╡ 401df970-8aa3-4f3a-bfe6-117d470b5fa1
let

    # Define the traces for the upper limit, lower limit, and the effective rate
    trace_upper = scatter(; x=EFFR_Montly.Months, y=EFFR_Montly.Upper, mode="lines", 
                          line_color="blue", name="Target rate: upper limit")

    trace_lower = scatter(; x=EFFR_Montly.Months, y=EFFR_Montly.Lower, 
						  mode="lines", line_width = "0", 
                          line_color="maroon", name="", 
                          fill="tonexty",
						  fillcolor="rgba(131, 90, 241, 0.35)"
						  #fillcolor="rgba(173, 216, 230, 0.99)"
	) # Light blue fill


	trace_upper = scatter(; x=EFFR_Montly.Months, y=EFFR_Montly.Upper, mode="lines", 
                          line_color="blue", name="Upper Limit: Target Rate Range")
	
	trace_headline = scatter(; x = x=EFFR_Montly.Months[109:188], y = How_Far.Headline[210:289] , mode = "lines", name = "Headline inflation", line_color="maroon")
	
	trace_lower2 = scatter(; x=EFFR_Montly.Months, y=EFFR_Montly.Lower.-0, mode="lines", 
                          line_color="black", name="Lower Limit: Target Rate Range")

    # Combine all traces into the figure
    fig05 = plot([trace_upper, trace_lower, trace_lower2, trace_headline])

    # Adjust the layout
    relayout!(fig05,
		xaxis_range = ["2018-01-01", "2024-01-01"],
        height=500,
        title_text="The Fed Target Rate Range and Headline Inflation",
        title_x=0.5,
        hovermode="x",
        legend=attr(x=0.05, y=0.95),
        yaxis_title="Percentage points",
        xaxis=attr(
            title="Monthly observations"
            # Additional axis settings can be uncommented and customized as needed
        )
    )

    # Display the figure
    fig05
end

# ╔═╡ cb2f0c66-30d1-4a44-bf6b-c4f706610dde
md"""
#### Supporting files: exercise 8 (Oil Prices & Fed ...)
"""

# ╔═╡ 49ed14dc-2915-4bf0-bfe9-08860a404123
Oil= CSV.read("Oil_CPI.csv", DataFrame);

# ╔═╡ 4ea050bf-b180-4e2d-b4d7-96a1831f8ce2
period9_1 = MonthlyDate(1971,1):Month(1):MonthlyDate(2023,9);

# ╔═╡ a4228555-5a74-41ed-96bd-afcec6a9d856
period9_2 = MonthlyDate(1998,1):Month(1):MonthlyDate(2023,9);

# ╔═╡ f342e1f8-03a6-42d4-b220-6fc8acb98b56
begin
	trace10_5 = scatter(;x = Date.(period9_2), y = Oil[325:end,3], name = "Oil", line_color = "Blue", 
						mode = "lines", line_width="2", marker_size ="6")
	
	
	trace10_6 = scatter(;x = Date.(period9_2), y = Oil[325:end,2], name = "CPI", line_color = "red", 
						mode = "lines", line_width="2", marker_size ="4", 
						yaxis = "y2")
	
	
	layout10_6 = Layout(
			height = 450,
			title_text = "Oil prices growth rate vs Headline inflation for the US (1998--2023)", title_x=0.5,
		
			hovermode="x",

			legend=attr( x=0.05, y=0.95),
		
			xaxis = attr(
                		title = "Monthly obervations",
                		tickformat = "%Y",
                		#hoverformat = "%Y-M%m",
                		#tick0 = "1989/19/01",
                		#dtick = "M48"
        						),
		
			yaxis1 = attr(
						title = "Oil",
						titlefont_color=  "blue",
						tickfont_color = "blue",
						#overlaying = "y",
						#side = "right",
						#yaxis1_tickvals = [1.5, 2.5, 3.5, 5.5],
						showgrid= false,
						zeroline=false,
    					#dtick = 5.6 / 7
						),
	

	  		yaxis2 = attr(
						title = "CPI",
						titlefont_color=  "red",
						tickfont_color = "red",
						overlaying = "y",
						side = "right",
						showgrid= true,
						zeroline=false,
						#yaxis2_range=[-27 , 27],
    					#dtick = 27 / 3
	  					)
		
						)

	p10_6 = plot([trace10_5,trace10_6],layout10_6)
end

# ╔═╡ ef782157-03be-4b91-b2e2-02df422ae736
plot(Oil[325:end,3] , Oil[325:end,2], mode = "markers", line_color = "darkred", text=period9_2,
	Layout(
		height = 580, width = 620,
		title_text = "Headline inflation vs Oil prices (1998--2023)", 
		title_x=0.5,
		xaxis_title = "Anual growth rates of Oil prices",
		yaxis_title = "Annual headline inflation"
	)
)

# ╔═╡ 0800e68d-ba07-40e4-aa0a-41767d0cfafa
md"""
#### Supporting cells: exercise 9 (Monetary Base versus ...)
"""

# ╔═╡ 26f09352-23fe-4f12-93ce-9298ea95ff61
Fed1 = CSV.read("Assets.csv", delim=";", DataFrame);

# ╔═╡ 98128fbc-f645-4e54-8025-2afe3cbe0c11
period4_1 = MonthlyDate(2003,1):Month(1):MonthlyDate(2022,8);

# ╔═╡ 83b2a77f-c6df-4c0d-aa5d-b82d193b6283
begin
	trace6_1 = scatter(;x = Date.(period4_1), y = Fed1[:,3], 
				name = "MB", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3, 
            	marker_color = "Blue")
	
	trace6_2 = scatter(;x = Date.(period4_1), y = Fed1[:,2], 
				name = "Assets", mode="markers+lines",
				marker_symbol="circle", marker_size="4",line_width= 0.3,
            	marker_color = "Red")

	layout6_2 = Layout(;
			
			height = 450,		
			title_text = "Total Assets vs Monetary Base in the US (2003--2022): Millions of Dollars", 
			title_x=0.5,
			hovermode="x",
			legend=attr( x=0.05, y=0.95),		
            xaxis = attr(
               title = "Monthly obervations",
               #tickformat = "%Y",
               #hoverformat = "%Y-M%m",
               tick0 = "2003/01/01",
               dtick = "M24" 
			   ),
		
        	#xaxis_range = [Date.(2003), Date.(2023)],
			xaxis_range = ["2002-10-01" , "2022-10-01"],
        	yaxis_title = "Millions of Dollars",
        	#yaxis_range=[-2, 2], 
        	titlefont_size = 16)

	p6_2 = plot([trace6_1, trace6_2], layout6_2)
end

# ╔═╡ 7a80f940-cc33-4ab5-8aaf-78718a718ce3
md"""
#### Supporting cells: exercise 11 (QE and financial markets)
"""

# ╔═╡ 171587c4-98f7-42c2-b9a3-eaf8b3a0dbfe
Shiller= CSV.read("Shiller_PE_Ratio.csv", delim=';', DataFrame);

# ╔═╡ 36663d1d-d5fa-4061-ac76-8138080cd860
Shiller2 = Shiller[1630:1741,:];

# ╔═╡ 15c89820-3b0d-4808-bf65-db3e0c93e0e3
period6_7 = MonthlyDate(2006,11):Month(1):MonthlyDate(2016, 2);

# ╔═╡ 92afd2fc-cd56-4925-b5c4-31cba7cd36a6
md"""
#### Supporting cells: exercise 12 (QE and inflation)
"""

# ╔═╡ ab708fcf-9cf4-4deb-8d1b-a69e5eaa87af
QE= CSV.read("QE.csv", delim=';', DataFrame);

# ╔═╡ 8ef02a3f-c864-4c12-8b90-eccc9f58fcb6
my_strings = QE[!,:Months] = string.(QE[:,:Months]);

# ╔═╡ 7d8c82db-bcac-400b-a313-5571d12aabf2
begin
	
	trace6_81 = scatter(;x = Date.(period6_7), y = Shiller2[:,2], 
				mode="markers+lines", name  = "Shiller PE Ratio", marker_symbol="circle",
				marker_size="5", line_width= 0.5, marker_color = "Blue")
	
	shapes6_81 = rect(["2008-12-01","2010-11-01","2011-09-01"], ["2010-04-01","2011-06-01","2014-10-01"],
                0, 1; fillcolor = "gray", opacity = 0.2, line_width = 0,
                xref = my_strings, yref = "paper")

	layout6_81 = Layout(;height = 450,
		
		title_text = "The Shiller PE Ratio for the USA (2007-2015)", title_x =0.5,
		
		hovermode="x",
		
		shapes = shapes6_81,
		
		annotations = [attr(x = "2009-07-01", y = 27, showarrow = false, text = "QE1"), 
			       	   attr(x = "2011-03-01", y = 27, showarrow = false, text = "QE2"),
					   attr(x = "2013-03-01", y = 27, showarrow = false, text = "QE3") ],
		
            xaxis = attr(
               title = "Monthly observations", 
               tickformat = "%Y",
               hoverformat = "%Y-M%m",
               tick0 = "2006/11/01",
               dtick = "M12"),
        xaxis_range = ["2006-10-01", "2016-03-01"],
		
        yaxis_title = "Shiller PE Ratio",
        yaxis_range=[12, 28], 
        titlefont_size = 16,
		dtick = 4)

	p6_81 = plot([trace6_81], layout6_81)
end

# ╔═╡ 53d1ddfb-b2ad-43f4-bbc4-b0943499d8e2
period6_6 = MonthlyDate(2006,1):Month(1):MonthlyDate(2015,4);

# ╔═╡ e8d359d9-6dba-4c50-bac0-af695012c64d
begin
	
	trace6_71 = scatter(;x = Date.(period6_6), y = QE[:,2], 
				mode="markers+lines", name  = "T5YIFR", marker_symbol="circle",
				marker_size="5", line_width= 0.3, marker_color = "Blue")
	
	trace6_72 = scatter(;x = Date.(period6_6), y = QE[:,3], 
				mode="markers+lines", name  = "CPI", marker_symbol="circle", 
				marker_size="5",line_width= 0.3, marker_color = "darkred")
	
	shapes6_72 = rect(["2008-12-01","2010-11-01","2011-09-01"], ["2010-04-01","2011-06-01","2014-10-01"],
                0, 1; fillcolor = "gray", opacity = 0.2, line_width = 0,
                xref = my_strings, yref = "paper")

	layout6_72 = Layout(; height = 450,
		
		title_text = "Inflation Expectations (T5YIFR) and Inflation Rate (CPI) for the USA (2006-2015)",  			title_x =0.5,
		
		hovermode="x",
		
		shapes = shapes6_72,
		
		annotations = [attr(x = "2009-07-01", y = 5.5, showarrow = false, text = "QE1"), 
			       	   attr(x = "2011-03-01", y = 5.5, showarrow = false, text = "QE2"),
					   attr(x = "2013-03-01", y = 5.5, showarrow = false, text = "QE3") ],
		
            xaxis = attr(
               title = "Monthly observations", 
               tickformat = "%Y",
               hoverformat = "%Y-M%m",
               tick0 = "2006/01/01",
               dtick = "M12",
        ),
        xaxis_range = ["2005-12-01", "2015-05-01"],
        yaxis_title = "Percentage points",
        #yaxis_range=[-2, 2], 
        titlefont_size = 16)

	p6_72 = plot([trace6_71, trace6_72], layout6_72)
end

# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
CSV = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
Dates = "ade2ca70-3891-5945-98fb-dc099432e06a"
HypertextLiteral = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
PeriodicalDates = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
PlutoPlotly = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
PlutoTeachingTools = "661c6b06-c737-4d37-b85c-46df65de6f69"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"

[compat]
CSV = "~0.10.4"
DataFrames = "~1.4.1"
HypertextLiteral = "~0.9.5"
PeriodicalDates = "~2.0.0"
PlutoPlotly = "~0.6.2"
PlutoTeachingTools = "~0.2.3"
PlutoUI = "~0.7.43"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

julia_version = "1.11.3"
manifest_format = "2.0"
project_hash = "28d4a050cfee0a8b6457301a4956238b4796323a"

[[deps.AbstractPlutoDingetjes]]
deps = ["Pkg"]
git-tree-sha1 = "6e1d2a35f2f90a4bc7c2ed98079b2ba09c35b83a"
uuid = "6e696c72-6542-2067-7265-42206c756150"
version = "1.3.2"

[[deps.ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"
version = "1.1.2"

[[deps.Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"
version = "1.11.0"

[[deps.Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"
version = "1.11.0"

[[deps.CSV]]
deps = ["CodecZlib", "Dates", "FilePathsBase", "InlineStrings", "Mmap", "Parsers", "PooledArrays", "PrecompileTools", "SentinelArrays", "Tables", "Unicode", "WeakRefStrings", "WorkerUtilities"]
git-tree-sha1 = "deddd8725e5e1cc49ee205a1964256043720a6c3"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.10.15"

[[deps.CodeTracking]]
deps = ["InteractiveUtils", "UUIDs"]
git-tree-sha1 = "7eee164f122511d3e4e1ebadb7956939ea7e1c77"
uuid = "da1fd8a2-8d9e-5ec2-8556-3022fb5608a2"
version = "1.3.6"

[[deps.CodecZlib]]
deps = ["TranscodingStreams", "Zlib_jll"]
git-tree-sha1 = "962834c22b66e32aa10f7611c08c8ca4e20749a9"
uuid = "944b1d66-785c-5afd-91f1-9de20f533193"
version = "0.7.8"

[[deps.ColorSchemes]]
deps = ["ColorTypes", "ColorVectorSpace", "Colors", "FixedPointNumbers", "PrecompileTools", "Random"]
git-tree-sha1 = "403f2d8e209681fcbd9468a8514efff3ea08452e"
uuid = "35d6a980-a343-548e-a6ea-1d62b119f2f4"
version = "3.29.0"

[[deps.ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "b10d0b65641d57b8b4d5e234446582de5047050d"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.5"

[[deps.ColorVectorSpace]]
deps = ["ColorTypes", "FixedPointNumbers", "LinearAlgebra", "Requires", "Statistics", "TensorCore"]
git-tree-sha1 = "a1f44953f2382ebb937d60dafbe2deea4bd23249"
uuid = "c3611d14-8923-5661-9e6a-0046d554d3a4"
version = "0.10.0"

    [deps.ColorVectorSpace.extensions]
    SpecialFunctionsExt = "SpecialFunctions"

    [deps.ColorVectorSpace.weakdeps]
    SpecialFunctions = "276daf66-3868-5448-9aa4-cd146d93841b"

[[deps.Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "362a287c3aa50601b0bc359053d5c2468f0e7ce0"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.11"

[[deps.Compat]]
deps = ["TOML", "UUIDs"]
git-tree-sha1 = "8ae8d32e09f0dcf42a36b90d4e17f5dd2e4c4215"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "4.16.0"
weakdeps = ["Dates", "LinearAlgebra"]

    [deps.Compat.extensions]
    CompatLinearAlgebraExt = "LinearAlgebra"

[[deps.CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"
version = "1.1.1+0"

[[deps.Crayons]]
git-tree-sha1 = "249fe38abf76d48563e2f4556bebd215aa317e15"
uuid = "a8cc5b0e-0ffa-5ad4-8c14-923d3ee1735f"
version = "4.1.1"

[[deps.DataAPI]]
git-tree-sha1 = "abe83f3a2f1b857aac70ef8b269080af17764bbe"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.16.0"

[[deps.DataFrames]]
deps = ["Compat", "DataAPI", "Future", "InvertedIndices", "IteratorInterfaceExtensions", "LinearAlgebra", "Markdown", "Missings", "PooledArrays", "PrettyTables", "Printf", "REPL", "Random", "Reexport", "SnoopPrecompile", "SortingAlgorithms", "Statistics", "TableTraits", "Tables", "Unicode"]
git-tree-sha1 = "d4f69885afa5e6149d0cab3818491565cf41446d"
uuid = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
version = "1.4.4"

[[deps.DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "1d0a14036acb104d9e89698bd408f63ab58cdc82"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.20"

[[deps.DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[deps.Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"
version = "1.11.0"

[[deps.DelimitedFiles]]
deps = ["Mmap"]
git-tree-sha1 = "9e2f36d3c96a820c678f2f1f1782582fcf685bae"
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"
version = "1.9.1"

[[deps.DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "2fb1e02f2b635d0845df5d7c167fec4dd739b00d"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.9.3"

[[deps.Downloads]]
deps = ["ArgTools", "FileWatching", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"
version = "1.6.0"

[[deps.FilePathsBase]]
deps = ["Compat", "Dates"]
git-tree-sha1 = "2ec417fc319faa2d768621085cc1feebbdee686b"
uuid = "48062228-2e41-5def-b9a4-89aafe57970f"
version = "0.9.23"
weakdeps = ["Mmap", "Test"]

    [deps.FilePathsBase.extensions]
    FilePathsBaseMmapExt = "Mmap"
    FilePathsBaseTestExt = "Test"

[[deps.FileWatching]]
uuid = "7b1f6079-737a-58dc-b8bc-7a2ca5c1b5ee"
version = "1.11.0"

[[deps.FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "05882d6995ae5c12bb5f36dd2ed3f61c98cbb172"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.5"

[[deps.Format]]
git-tree-sha1 = "9c68794ef81b08086aeb32eeaf33531668d5f5fc"
uuid = "1fa38f19-a742-5d3f-a2b9-30dd87b9d5f8"
version = "1.3.7"

[[deps.Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"
version = "1.11.0"

[[deps.HashArrayMappedTries]]
git-tree-sha1 = "2eaa69a7cab70a52b9687c8bf950a5a93ec895ae"
uuid = "076d061b-32b6-4027-95e0-9a2c6f6d7e74"
version = "0.2.0"

[[deps.Hyperscript]]
deps = ["Test"]
git-tree-sha1 = "179267cfa5e712760cd43dcae385d7ea90cc25a4"
uuid = "47d2ed2b-36de-50cf-bf87-49c2cf4b8b91"
version = "0.0.5"

[[deps.HypertextLiteral]]
deps = ["Tricks"]
git-tree-sha1 = "7134810b1afce04bbc1045ca1985fbe81ce17653"
uuid = "ac1192a8-f4b3-4bfe-ba22-af5b92cd3ab2"
version = "0.9.5"

[[deps.IOCapture]]
deps = ["Logging", "Random"]
git-tree-sha1 = "b6d6bfdd7ce25b0f9b2f6b3dd56b2673a66c8770"
uuid = "b5f81e59-6552-4d32-b1f0-c071b021bf89"
version = "0.2.5"

[[deps.InlineStrings]]
git-tree-sha1 = "45521d31238e87ee9f9732561bfee12d4eebd52d"
uuid = "842dd82b-1e85-43dc-bf29-5d0ee9dffc48"
version = "1.4.2"

    [deps.InlineStrings.extensions]
    ArrowTypesExt = "ArrowTypes"
    ParsersExt = "Parsers"

    [deps.InlineStrings.weakdeps]
    ArrowTypes = "31f734f8-188a-4ce0-8406-c8a06bd891cd"
    Parsers = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"

[[deps.InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"
version = "1.11.0"

[[deps.InvertedIndices]]
git-tree-sha1 = "6da3c4316095de0f5ee2ebd875df8721e7e0bdbe"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.3.1"

[[deps.IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[deps.JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "31e996f0a15c7b280ba9f76636b3ff9e2ae58c9a"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.4"

[[deps.JuliaInterpreter]]
deps = ["CodeTracking", "InteractiveUtils", "Random", "UUIDs"]
git-tree-sha1 = "4bf4b400a8234cff0f177da4a160a90296159ce9"
uuid = "aa1ae85d-cabe-5617-a682-6adf51b2e16a"
version = "0.9.41"

[[deps.LaTeXStrings]]
git-tree-sha1 = "dda21b8cbd6a6c40d9d02a73230f9d70fed6918c"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.4.0"

[[deps.Latexify]]
deps = ["Format", "InteractiveUtils", "LaTeXStrings", "MacroTools", "Markdown", "OrderedCollections", "Requires"]
git-tree-sha1 = "cd714447457c660382fe634710fb56eb255ee42e"
uuid = "23fbe1c1-3f47-55db-b15f-69d7ec21a316"
version = "0.16.6"

    [deps.Latexify.extensions]
    DataFramesExt = "DataFrames"
    SparseArraysExt = "SparseArrays"
    SymEngineExt = "SymEngine"

    [deps.Latexify.weakdeps]
    DataFrames = "a93c6f00-e57d-5684-b7b6-d8193f3e46c0"
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"
    SymEngine = "123dc426-2d89-5057-bbad-38513e3affd8"

[[deps.LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"
version = "0.6.4"

[[deps.LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"
version = "8.6.0+0"

[[deps.LibGit2]]
deps = ["Base64", "LibGit2_jll", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"
version = "1.11.0"

[[deps.LibGit2_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll"]
uuid = "e37daf67-58a4-590a-8e99-b0245dd2ffc5"
version = "1.7.2+0"

[[deps.LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"
version = "1.11.0+1"

[[deps.Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"
version = "1.11.0"

[[deps.LinearAlgebra]]
deps = ["Libdl", "OpenBLAS_jll", "libblastrampoline_jll"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"
version = "1.11.0"

[[deps.Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"
version = "1.11.0"

[[deps.LoweredCodeUtils]]
deps = ["JuliaInterpreter"]
git-tree-sha1 = "688d6d9e098109051ae33d126fcfc88c4ce4a021"
uuid = "6f1432cf-f94c-5a45-995e-cdbf5db27b0b"
version = "3.1.0"

[[deps.MIMEs]]
git-tree-sha1 = "1833212fd6f580c20d4291da9c1b4e8a655b128e"
uuid = "6c6e2e6c-3030-632d-7369-2d6c69616d65"
version = "1.0.0"

[[deps.MacroTools]]
git-tree-sha1 = "72aebe0b5051e5143a079a4685a46da330a40472"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.15"

[[deps.Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"
version = "1.11.0"

[[deps.MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"
version = "2.28.6+0"

[[deps.Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "ec4f7fbeab05d7747bdf98eb74d130a2a2ed298d"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.2.0"

[[deps.Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"
version = "1.11.0"

[[deps.MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"
version = "2023.12.12"

[[deps.NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"
version = "1.2.0"

[[deps.OpenBLAS_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "Libdl"]
uuid = "4536629a-c528-5b80-bd46-f80d51c5b363"
version = "0.3.27+1"

[[deps.OrderedCollections]]
git-tree-sha1 = "cc4054e898b852042d7b503313f7ad03de99c3dd"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.8.0"

[[deps.Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "34c0e9ad262e5f7fc75b10a9952ca7692cfc5fbe"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.3"

[[deps.Parsers]]
deps = ["Dates", "PrecompileTools", "UUIDs"]
git-tree-sha1 = "8489905bcdbcfac64d1daa51ca07c0d8f0283821"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "2.8.1"

[[deps.PeriodicalDates]]
deps = ["Dates", "Printf", "RecipesBase"]
git-tree-sha1 = "e616941f8093e50a373e7d51875143213f82eca4"
uuid = "276e7ca9-e0d7-440b-97bc-a6ae82f545b1"
version = "2.0.0"

[[deps.Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "FileWatching", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "Random", "SHA", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
version = "1.11.0"
weakdeps = ["REPL"]

    [deps.Pkg.extensions]
    REPLExt = "REPL"

[[deps.PlotlyBase]]
deps = ["ColorSchemes", "Dates", "DelimitedFiles", "DocStringExtensions", "JSON", "LaTeXStrings", "Logging", "Parameters", "Pkg", "REPL", "Requires", "Statistics", "UUIDs"]
git-tree-sha1 = "56baf69781fc5e61607c3e46227ab17f7040ffa2"
uuid = "a03496cd-edff-5a9b-9e67-9cda94a718b5"
version = "0.8.19"

[[deps.PlutoHooks]]
deps = ["InteractiveUtils", "Markdown", "UUIDs"]
git-tree-sha1 = "072cdf20c9b0507fdd977d7d246d90030609674b"
uuid = "0ff47ea0-7a50-410d-8455-4348d5de0774"
version = "0.0.5"

[[deps.PlutoLinks]]
deps = ["FileWatching", "InteractiveUtils", "Markdown", "PlutoHooks", "Revise", "UUIDs"]
git-tree-sha1 = "8f5fa7056e6dcfb23ac5211de38e6c03f6367794"
uuid = "0ff47ea0-7a50-410d-8455-4348d5de0420"
version = "0.1.6"

[[deps.PlutoPlotly]]
deps = ["AbstractPlutoDingetjes", "Artifacts", "ColorSchemes", "Colors", "Dates", "Downloads", "HypertextLiteral", "InteractiveUtils", "LaTeXStrings", "Markdown", "Pkg", "PlotlyBase", "PrecompileTools", "Reexport", "ScopedValues", "Scratch", "TOML"]
git-tree-sha1 = "9ebe25fc4703d4112cc418834d5e4c9a4b29087d"
uuid = "8e989ff0-3d88-8e9f-f020-2b208a939ff0"
version = "0.6.2"

    [deps.PlutoPlotly.extensions]
    PlotlyKaleidoExt = "PlotlyKaleido"
    UnitfulExt = "Unitful"

    [deps.PlutoPlotly.weakdeps]
    PlotlyKaleido = "f2990250-8cf9-495f-b13a-cce12b45703c"
    Unitful = "1986cc42-f94f-5a68-af5c-568840ba703d"

[[deps.PlutoTeachingTools]]
deps = ["Downloads", "HypertextLiteral", "LaTeXStrings", "Latexify", "Markdown", "PlutoLinks", "PlutoUI", "Random"]
git-tree-sha1 = "5d9ab1a4faf25a62bb9d07ef0003396ac258ef1c"
uuid = "661c6b06-c737-4d37-b85c-46df65de6f69"
version = "0.2.15"

[[deps.PlutoUI]]
deps = ["AbstractPlutoDingetjes", "Base64", "ColorTypes", "Dates", "FixedPointNumbers", "Hyperscript", "HypertextLiteral", "IOCapture", "InteractiveUtils", "JSON", "Logging", "MIMEs", "Markdown", "Random", "Reexport", "URIs", "UUIDs"]
git-tree-sha1 = "7e71a55b87222942f0f9337be62e26b1f103d3e4"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.61"

[[deps.PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "36d8b4b899628fb92c2749eb488d884a926614d3"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.4.3"

[[deps.PrecompileTools]]
deps = ["Preferences"]
git-tree-sha1 = "5aa36f7049a63a1528fe8f7c3f2113413ffd4e1f"
uuid = "aea7be01-6a6a-4083-8856-8a6e6704d82a"
version = "1.2.1"

[[deps.Preferences]]
deps = ["TOML"]
git-tree-sha1 = "9306f6085165d270f7e3db02af26a400d580f5c6"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.4.3"

[[deps.PrettyTables]]
deps = ["Crayons", "LaTeXStrings", "Markdown", "PrecompileTools", "Printf", "Reexport", "StringManipulation", "Tables"]
git-tree-sha1 = "1101cd475833706e4d0e7b122218257178f48f34"
uuid = "08abe8d2-0d0c-5749-adfa-8a2ac140af0d"
version = "2.4.0"

[[deps.Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"
version = "1.11.0"

[[deps.REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "StyledStrings", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"
version = "1.11.0"

[[deps.Random]]
deps = ["SHA"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"
version = "1.11.0"

[[deps.RecipesBase]]
deps = ["PrecompileTools"]
git-tree-sha1 = "5c3d09cc4f31f5fc6af001c250bf1278733100ff"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.3.4"

[[deps.Reexport]]
git-tree-sha1 = "45e428421666073eab6f2da5c9d310d99bb12f9b"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.2.2"

[[deps.Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "838a3a4188e2ded87a4f9f184b4b0d78a1e91cb7"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.3.0"

[[deps.Revise]]
deps = ["CodeTracking", "FileWatching", "JuliaInterpreter", "LibGit2", "LoweredCodeUtils", "OrderedCollections", "REPL", "Requires", "UUIDs", "Unicode"]
git-tree-sha1 = "9bb80533cb9769933954ea4ffbecb3025a783198"
uuid = "295af30f-e4ad-537b-8983-00126c2a3abe"
version = "3.7.2"

    [deps.Revise.extensions]
    DistributedExt = "Distributed"

    [deps.Revise.weakdeps]
    Distributed = "8ba89e20-285c-5b6f-9357-94700520ee1b"

[[deps.SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"
version = "0.7.0"

[[deps.ScopedValues]]
deps = ["HashArrayMappedTries", "Logging"]
git-tree-sha1 = "1147f140b4c8ddab224c94efa9569fc23d63ab44"
uuid = "7e506255-f358-4e82-b7e4-beb19740aa63"
version = "1.3.0"

[[deps.Scratch]]
deps = ["Dates"]
git-tree-sha1 = "3bac05bc7e74a75fd9cba4295cde4045d9fe2386"
uuid = "6c6a2e73-6563-6170-7368-637461726353"
version = "1.2.1"

[[deps.SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "712fb0231ee6f9120e005ccd56297abbc053e7e0"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.4.8"

[[deps.Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"
version = "1.11.0"

[[deps.SnoopPrecompile]]
deps = ["Preferences"]
git-tree-sha1 = "e760a70afdcd461cf01a575947738d359234665c"
uuid = "66db9d55-30c0-4569-8b51-7e840670fc0c"
version = "1.0.3"

[[deps.Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"
version = "1.11.0"

[[deps.SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "66e0a8e672a0bdfca2c3f5937efb8538b9ddc085"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.2.1"

[[deps.Statistics]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "ae3bb1eb3bba077cd276bc5cfc337cc65c3075c0"
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"
version = "1.11.1"

    [deps.Statistics.extensions]
    SparseArraysExt = ["SparseArrays"]

    [deps.Statistics.weakdeps]
    SparseArrays = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[deps.StringManipulation]]
deps = ["PrecompileTools"]
git-tree-sha1 = "725421ae8e530ec29bcbdddbe91ff8053421d023"
uuid = "892a3eda-7b42-436c-8928-eab12a02cf0e"
version = "0.4.1"

[[deps.StyledStrings]]
uuid = "f489334b-da3d-4c2e-b8f0-e476e12c162b"
version = "1.11.0"

[[deps.TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"
version = "1.0.3"

[[deps.TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[deps.Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "OrderedCollections", "TableTraits"]
git-tree-sha1 = "598cd7c1f68d1e205689b1c2fe65a9f85846f297"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.12.0"

[[deps.Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"
version = "1.10.0"

[[deps.TensorCore]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "1feb45f88d133a655e001435632f019a9a1bcdb6"
uuid = "62fd8b95-f654-4bbd-a8a5-9c27f68ccd50"
version = "0.1.1"

[[deps.Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"
version = "1.11.0"

[[deps.TranscodingStreams]]
git-tree-sha1 = "0c45878dcfdcfa8480052b6ab162cdd138781742"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.11.3"

[[deps.Tricks]]
git-tree-sha1 = "6cae795a5a9313bbb4f60683f7263318fc7d1505"
uuid = "410a4b4d-49e4-4fbc-ab6d-cb71b17b3775"
version = "0.1.10"

[[deps.URIs]]
git-tree-sha1 = "67db6cc7b3821e19ebe75791a9dd19c9b1188f2b"
uuid = "5c2747f8-b7ea-4ff2-ba2e-563bfd36b1d4"
version = "1.5.1"

[[deps.UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"
version = "1.11.0"

[[deps.UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[deps.Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"
version = "1.11.0"

[[deps.WeakRefStrings]]
deps = ["DataAPI", "InlineStrings", "Parsers"]
git-tree-sha1 = "b1be2855ed9ed8eac54e5caff2afcdb442d52c23"
uuid = "ea10d353-3f73-51f8-a26c-33c1cb351aa5"
version = "1.4.2"

[[deps.WorkerUtilities]]
git-tree-sha1 = "cd1659ba0d57b71a464a29e64dbc67cfe83d54e7"
uuid = "76eceee3-57b5-4d4a-8e66-0e911cebbf60"
version = "1.6.1"

[[deps.Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"
version = "1.2.13+1"

[[deps.libblastrampoline_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850b90-86db-534c-a0d3-1478176c7d93"
version = "5.11.0+0"

[[deps.nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"
version = "1.59.0+0"

[[deps.p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
version = "17.4.0+2"
"""

# ╔═╡ Cell order:
# ╟─10d33d47-0727-4c01-acdb-71949077bd2e
# ╟─943edb36-acc8-4b2a-94ab-c544f9eb279b
# ╟─1245aef9-18ce-42f5-af82-c2e6cce70798
# ╟─55188a89-8172-4bb9-9569-765a9f277d0f
# ╟─4618a663-f77e-4ccc-9b00-e84dbd5beb34
# ╠═a145e2fe-a903-11eb-160f-df7ea83fa3e6
# ╟─72d45107-22d6-4663-a9b8-c540a4c7411b
# ╟─116306a9-5f2d-4b96-9124-584dc9835a8a
# ╟─19741208-c2bd-455a-a19f-74521a8c56bf
# ╟─9877083b-a80c-4069-89e4-84638fe6ecbb
# ╟─0be26b61-30ce-4128-b2b1-b4eb955244d1
# ╟─cfc72d68-3d45-4673-81a2-7032896fbe89
# ╟─be5df6ed-9417-4ed0-862c-8f337b80084e
# ╟─f0a318ef-47b0-4fa3-aeb1-e7d1548fd214
# ╟─b189ebff-1493-4d95-98d2-99d6f55884fe
# ╠═c99c90e1-e91f-4682-a190-495d2cbd0b06
# ╠═98e7e1ad-2964-41e5-9162-cd077978ee15
# ╠═39480dd4-4e0e-4e5f-9a5c-74f426b9193a
# ╟─f83dd4f0-0312-4bb0-98b0-67eeab7177e4
# ╠═4bf99794-a645-40c7-b51b-128fce7814df
# ╠═baab71f8-e0be-43b5-83f4-708e4e575ebc
# ╟─719dbca7-f3f1-4827-bfba-d9813979c42a
# ╟─cdeb5418-39ca-402c-81fd-1a4e42c2e105
# ╟─02126494-1ba6-47f8-8c10-9c4d34e20d57
# ╟─b7749c4f-ade7-46ec-a95f-ea1349b86956
# ╟─84cee9af-0999-4659-a537-85c89f9182cc
# ╟─acce2747-50a8-4531-a4e1-d7bd971f2210
# ╟─c144c3fe-d49f-4243-bdc2-34ba8af88673
# ╟─1a073295-059c-4815-b3bd-450f5f88c8d1
# ╟─99fa722b-bef4-48d7-bd4a-f0459f15eed3
# ╟─f4706381-35df-44dd-aea2-83220a94c9d4
# ╟─8eeea220-e319-45da-a774-7cf1475f2424
# ╟─4f82c4f5-dab7-4ef4-a1c6-bef80ac2fe64
# ╟─71bbd822-914c-4ea6-93c5-0b3ad86000ca
# ╟─10234ea1-6528-4fd2-bdae-3ca5a36e405b
# ╟─76d208b1-80b7-4a8c-96c6-2de15b5e89e8
# ╟─1254cc13-8ac0-468c-bbc5-e838018fa1de
# ╟─e7f794fa-ede6-4391-b703-5488fd739f29
# ╟─15b3a15f-a0a0-4623-b6ec-43dc33bbe1e3
# ╟─d2a12416-9d54-479c-956d-2d8869024727
# ╟─3323975f-e90c-4a39-87d4-2ae9c676e24a
# ╟─a4aecd27-1376-4f74-98b6-10502c93e1d0
# ╟─d865e28a-5db4-4b39-a7e6-353d380b9e1a
# ╟─32c928dd-ca87-4db5-aa28-222e43a7a3cb
# ╟─5d7ad15e-3057-43c8-913c-67f6a71338b7
# ╟─f54d0589-0567-4b75-91f2-d9bf223c5fb6
# ╟─2bd8b31a-8fa2-4404-816e-158df0afcc7d
# ╟─30f40a81-32cb-4d2b-b3c5-b0b155648283
# ╟─a9095027-f033-4253-9269-105e51887424
# ╟─2c3766cf-1349-4e99-a3fd-62ad6e5fe18a
# ╟─d6a5e159-8b97-4660-8e00-cd251e5d0216
# ╟─c5363210-26f9-40b1-9a1a-9ac80ffac890
# ╟─3193d821-a4e1-404f-a57e-8c471714bb9d
# ╟─1471566f-469f-421b-8671-abc49e040bc1
# ╟─624e0704-a138-4ddd-8086-078d98a84a41
# ╟─e7fd66aa-cb38-4b7f-92af-aa4fd15549b2
# ╟─fe0e5144-02b9-4df3-84ad-3298a8703baa
# ╟─6e20a099-8d46-4a35-83ce-e62c977a6b3d
# ╟─65da15c3-dcad-4e38-b0aa-bf9dd456285d
# ╟─74529060-483d-4afe-8d58-155c5f0b4b1e
# ╟─ced280ff-d8c1-414e-8855-387727d5dfe2
# ╟─85cb81f0-8e2f-4142-9fdc-b8839d2a6cea
# ╟─5e6f355c-cb29-416f-8799-0234fbc87f1f
# ╟─5bb6a994-ac2d-44cd-89c9-c3c3fbdca100
# ╟─14396951-fcfa-41eb-822a-f673cd830565
# ╟─6f384fbf-b581-4eb8-9ce4-026714d99537
# ╟─74d6dd0f-8cf0-45dc-8f5a-250560e8a349
# ╟─10417add-96dd-427b-825a-dc558f710807
# ╟─3e9327e9-b113-4468-b620-4ae8552c214d
# ╟─8007d3dd-d6e1-405c-9018-56cd30004af7
# ╟─b7cb4d68-1e36-4d1f-acaa-0855d2127ef8
# ╟─7641617b-48f0-4ec2-8504-3d655d0134ad
# ╟─b0b75354-1c37-4f06-afe3-5b1d420ae245
# ╟─164e38a5-1368-4456-a909-dbe8677ebcb8
# ╟─4550ba42-1a49-4fa9-932b-e4d34210233f
# ╟─88bab2ef-32ba-49b0-a5e0-36dad20e39e2
# ╟─869004a1-039e-41e4-a61f-453c25406ddc
# ╟─f4fe404c-ca57-4682-8a51-3c08abedfb36
# ╟─60e62656-e863-4595-9d94-4e7d0cf6d935
# ╟─88675aac-0d69-4028-aee1-c0e0c22ab5e2
# ╟─2283145f-b909-4b23-8292-4bb660f303d1
# ╟─642da752-5c76-482e-97e5-a9964e894868
# ╟─08bf1683-0c0d-4c52-a446-6f8be4135aed
# ╟─d08dc62f-8951-4e36-8818-c325316ece23
# ╟─bf337699-7647-4538-85e1-ed2fd9d05269
# ╟─5b1919fb-f5f5-4682-a1e0-1c767825c902
# ╟─baf57f6e-c2db-49f5-b65e-11656176372c
# ╟─a8dbf8b7-1f6a-48c1-b8e8-c73062f72277
# ╟─1d323377-a98c-4a93-977f-7796acede851
# ╟─ba7a9892-dc53-4c9a-867f-17d0812ec97a
# ╟─401df970-8aa3-4f3a-bfe6-117d470b5fa1
# ╟─c3ab833b-0d82-40b3-9553-d90c32c369d8
# ╟─88a9d07a-d222-46b3-bcae-d19e4ac78017
# ╟─c93efdd7-1ae9-4989-b5cb-cfdb4347086d
# ╟─8f643bb9-8826-421f-8590-b440c65101eb
# ╟─dca53198-44ed-402e-8a68-401a6006d1f9
# ╟─2b7f9399-1606-4374-abfb-abf9e7ab2e90
# ╟─0223f27f-532b-4b75-af02-817fe5f44ed6
# ╟─268c2d4f-0211-4ff5-a054-5dbe4c4b3078
# ╟─f342e1f8-03a6-42d4-b220-6fc8acb98b56
# ╟─ef782157-03be-4b91-b2e2-02df422ae736
# ╟─b04f7af0-7e7c-45a5-ad82-78e866348567
# ╟─d710baca-f026-4e3d-926c-145594eedc81
# ╟─1a8a1ae4-6ecd-4a91-a482-3024ca11fb98
# ╟─69621e4b-5163-40a4-a70c-429d8afe6ff5
# ╟─c842b2da-1bf5-43db-9bc0-a8af8be64b9c
# ╟─7e7e4292-dfc8-41b4-a5a0-d7646f8fb495
# ╟─83b2a77f-c6df-4c0d-aa5d-b82d193b6283
# ╟─e547d364-216c-4165-982f-1e04c9a0945c
# ╟─01875faf-3231-4bb8-899d-0dd4e569ddc9
# ╟─63efdb6e-dff1-420f-a99d-90cece86bac2
# ╟─1a1d2e86-9436-4fe3-8dbd-74509f439a98
# ╟─9c396c2d-9512-4fab-a49f-7883ddb1d98c
# ╟─a5654312-d9e7-4d86-8663-6e4b4ea4d9b7
# ╟─dde1d26c-da2e-46ad-85cb-fcb03dc4b126
# ╟─5ba94526-2197-42ff-a569-038a07e83840
# ╟─660066db-af49-4473-aeff-161aedee0aba
# ╟─7d8c82db-bcac-400b-a313-5571d12aabf2
# ╟─06beba5f-e3af-4649-8d19-48a7036ebe01
# ╟─98338f07-e909-4c20-b2db-bf561db17b81
# ╟─ed79aa0d-e286-40dd-aa8f-19804ca94b55
# ╟─94c6354c-4f89-4982-8d18-677250f6380d
# ╟─def794bb-0c9e-4132-9300-d597fb29fd64
# ╟─10dad751-127a-44b3-a3cb-a22644049964
# ╟─e8d359d9-6dba-4c50-bac0-af695012c64d
# ╟─449374a2-0df7-4b8c-9286-79764bc5529f
# ╟─45b82b1a-262c-4668-879a-998acd7e160a
# ╟─09b4be5f-cb7e-4ffa-bbe2-a4e5b2d4fe66
# ╟─9fa87191-7c90-4738-a45a-acd929c8bd1b
# ╟─7c178839-3e3f-4013-8351-86efde9d1b5b
# ╟─953fc3de-48af-42f6-946b-2225618c093b
# ╟─ccb36e23-9fa5-43c4-a451-275609b4892b
# ╟─4c805ee4-e91e-4d88-bffc-e19e42868ef5
# ╟─cb9d826b-b609-49d5-ad60-a15efc8cce1d
# ╟─e7a2be16-2b78-4527-b799-6c208854ff5a
# ╟─cd8e9140-5696-4019-b208-d6430e68f978
# ╟─f05055c8-cb5f-4d28-91c0-d84d103efe43
# ╟─5d20e28c-8365-470b-9c41-0471e2c65ad8
# ╟─a5c866f0-3546-4934-ae84-9de9bc8e7f64
# ╟─b2d64bfc-2fbd-4677-8a17-dad2cc94b8e9
# ╟─202b8d36-8f6f-40ad-a173-e9213996ea5e
# ╟─fd598141-c9e6-402d-97f9-bab7e9f8b4dc
# ╟─cb2f0c66-30d1-4a44-bf6b-c4f706610dde
# ╟─49ed14dc-2915-4bf0-bfe9-08860a404123
# ╟─4ea050bf-b180-4e2d-b4d7-96a1831f8ce2
# ╟─a4228555-5a74-41ed-96bd-afcec6a9d856
# ╟─0800e68d-ba07-40e4-aa0a-41767d0cfafa
# ╟─26f09352-23fe-4f12-93ce-9298ea95ff61
# ╟─98128fbc-f645-4e54-8025-2afe3cbe0c11
# ╟─7a80f940-cc33-4ab5-8aaf-78718a718ce3
# ╟─171587c4-98f7-42c2-b9a3-eaf8b3a0dbfe
# ╟─36663d1d-d5fa-4061-ac76-8138080cd860
# ╟─15c89820-3b0d-4808-bf65-db3e0c93e0e3
# ╟─8ef02a3f-c864-4c12-8b90-eccc9f58fcb6
# ╟─92afd2fc-cd56-4925-b5c4-31cba7cd36a6
# ╟─ab708fcf-9cf4-4deb-8d1b-a69e5eaa87af
# ╟─53d1ddfb-b2ad-43f4-bbc4-b0943499d8e2
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
